/* MiscMergeExpression.h created by akira on Fri 27-Sep-2002 */

#import <Foundation/Foundation.h>

@class MiscMergeEngine;

#define MiscMergeConditionalOperator 16
#define MiscMergeMathOperator 32

typedef enum _MiscMergeOperator {
    MiscMergeOperatorNone = 0,

    MiscMergeOperatorEqual = MiscMergeConditionalOperator,
    MiscMergeOperatorNotEqual,
    MiscMergeOperatorLessThan,
    MiscMergeOperatorLessThanOrEqual,
    MiscMergeOperatorGreaterThan,
    MiscMergeOperatorGreaterThanOrEqual,

    MiscMergeOperatorAdd = MiscMergeMathOperator,
    MiscMergeOperatorSubtract,
    MiscMergeOperatorMultiply,
    MiscMergeOperatorDivide,
    MiscMergeOperatorModulus
} MiscMergeOperator;


@interface MiscMergeExpression : NSObject

- (id)evaluateWithEngine:(MiscMergeEngine *)anEngine;
- (BOOL)evaluateAsBoolWithEngine:(MiscMergeEngine *)anEngine;
- (int)evaluateAsIntWithEngine:(MiscMergeEngine *)anEngine;

@end


@interface MiscMergeValueExpression : MiscMergeExpression
{
    NSString *valueName;
    int quotes;
}

+ (MiscMergeValueExpression *)valueName:(NSString *)string quotes:(int)number;

@end


@interface MiscMergeUnaryOpExpression : MiscMergeExpression
{
    MiscMergeExpression *expression;
}

+ (MiscMergeUnaryOpExpression *)expression:(MiscMergeExpression *)expression;

@end

@interface MiscMergeNegativeExpression : MiscMergeUnaryOpExpression
@end

@interface MiscMergeNegateExpression : MiscMergeUnaryOpExpression
@end


@interface MiscMergeBinaryOpExpression : MiscMergeExpression
{
    MiscMergeExpression *leftExpression;
    MiscMergeExpression *rightExpression;
    MiscMergeOperator operator; 
}

+ (MiscMergeBinaryOpExpression *)leftExpression:(MiscMergeExpression *)expression
                                       operator:(MiscMergeOperator)operator
                                rightExpression:(MiscMergeExpression *)expression;

@end

@interface MiscMergeMathExpression : MiscMergeBinaryOpExpression
@end

@interface MiscMergeConditionalExpression : MiscMergeBinaryOpExpression
@end


@interface MiscMergeGroupExpression : MiscMergeExpression
{
    NSMutableArray *expressions;
}

+ (MiscMergeGroupExpression *)expression:(MiscMergeExpression *)expression
                           andExpression:(MiscMergeExpression *)expression;

- (void)addExpression:(MiscMergeExpression *)expression;

@end

@interface MiscMergeAndExpression : MiscMergeGroupExpression
@end

@interface MiscMergeOrExpression : MiscMergeGroupExpression
@end
