/* MiscMergeFunctions.m created by akira on Tue 03-Sep-2002 */
/*-
* Copyright (c) 2002
 *      Blacksmith, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Blacksmith, Inc.
 *      and its contributors.
 * 4. Neither the name Blacksmith nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY BLACKSMITH AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL BLACKSMITH OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#import "MiscMergeFunctions.h"
#import <Foundation/NSValue.h>
#import "NSString+MiscAdditions.h"

BOOL MMBoolValueOfObject(id anObject)
{
    if ( MMIsObjectANumber(anObject) )
        return MMDoubleValueForObject(anObject) != 0;
    else if ( MMIsObjectAString(anObject) )
        return ([anObject length] > 0);
    else
        return (anObject != nil);
}

int MMIntegerValueOfObject(id anObject)
{
    if ([anObject isKindOfClass:[NSNumber class]])
        return [anObject intValue];
    
    if ([anObject isKindOfClass:[NSString class]]) {
        NSScanner *scanner = [NSScanner scannerWithString:anObject];
        int intValue;
        [scanner scanInt:&intValue];
        return intValue;
    }

    return 0;
}

double MMDoubleValueForObject(id anObject)
{
    if ( MMIsObjectANumber(anObject) )
        return [anObject doubleValue];
    else if ( MMIsObjectAString(anObject) )
        return (double)([anObject length] > 0);
    else
        return (double)(anObject != nil);
}

BOOL MMIsObjectANumber(id anObject)
{
    if ([anObject isKindOfClass:[NSNumber class]])
        return YES;
    
    if ([anObject isKindOfClass:[NSString class]]) {
        NSScanner *scanner = [NSScanner scannerWithString:anObject];
        float  floatValue;
        return [scanner scanFloat:&floatValue];
    }

    return NO;
}

BOOL MMIsObjectAString(id anObject)
{
    return [anObject isKindOfClass:[NSString class]];
}

BOOL MMIsBooleanTrueString(NSString *anObject)
{
    NSString *string = [anObject lowercaseString];
    return [string isEqualToString:@"yes"] || [string isEqualToString:@"true"];
}


NSComparisonResult MMCompareFloats(float num1, float num2)
{
    if (num1 < num2) return NSOrderedAscending;
    if (num1 > num2) return NSOrderedDescending;
    return NSOrderedSame;
}


Class MMCommonAnscestorClass(id obj1, id obj2)
{
    Class currSuperclass = [obj1 class];

    for (; currSuperclass != nil; currSuperclass = [currSuperclass superclass]) {
        Class currTestClass = [obj2 class];
        
        for (; currTestClass != nil; currTestClass = [currTestClass superclass]) {
            if (currTestClass == currSuperclass)
                return currSuperclass;
        }
    }

    return nil;
}


NSString *MMStringByTrimmingCommandSpace(NSString *string)
{
    if ( [string isBlank] ) {
        return @"";
    }
    else {
        NSCharacterSet *whitespaceSet = [NSCharacterSet whitespaceCharacterSet];
        NSCharacterSet *newlineSet = [NSCharacterSet newlineCharacterSet];
        int start = 0, end = [string length]-1;
        int i;

        for ( i = 0; i <= end; i++ ) {
            unichar character = [string characterAtIndex:i];

            if ( ![whitespaceSet characterIsMember:character] ) {
                if ( [newlineSet characterIsMember:character] )
                    start = i+1;
                break;
            }
        }

        for ( i = end; i >= start; i-- ) {
            unichar character = [string characterAtIndex:i];

            if ( ![whitespaceSet characterIsMember:character] ) {
                if ( [newlineSet characterIsMember:character] )
                    end = i;
                break;
            }
        }

        return [string substringWithRange:NSMakeRange(start, end - start + 1)];
    }
}